if($pref::server::trading::enabled $= "")
	$pref::server::trading::enabled = true;

function tradeObject::checkTradeExpire(%this)
{
	if(!%this.targetAccepted)
	{
		%this.creator.chatMessage("Trade offer has timed out.");
		%this.target.chatMessage("Trade offer has timed out.");
		%this.delete();
		return;
	}
}

function tradeObject::expireTrade(%this)
{
	%this.creator.chatMessage("Trade has timed out.");
	%this.target.chatMessage("Trade has timed out.");
	%this.delete();
	return;
}

function servercmdTrade(%client, %name0, %name1, %name2, %name3)
{
	if(!$pref::server::trading::enabled)
	{
		%client.chatMessage("Trading is currently disabled.");
		return;
	}
	
	if((%client.lastTradeRequest > $sim::time - 40 && !%client.isAdmin) || isObject(%client.trade))
	{
		%client.chatMessage("Wait a minute before sending another request.");
		return;
	}
	
	%name = trim(%name0 SPC %name1 SPC %name2 SPC %name3);
	
	if(%name $= "")
	{
		%client.chatMessage("You must specify a name.");
		return;
	}
	
	%target = findclientbyname(%name);
	
	if(!isObject(%target))
	{
		%client.chatMessage("Nobody by that name was found.");
		return;
	}
	
	if(isObject(%target.trade))
	{
		%client.chatMessage("That player is already busy with another trade.");
		return;
	}
	
	%client.lastTradeRequest = $sim::time;
	
	%tradeObject = new ScriptObject(tradeObject)
	{
		creator = %client;
		target = %target;
		targetAccepted = false;
		
		creatorItemsOfferred = 0;
		targetItemsOfferred = 0;
		
		creatorConfirmed = false;
		targetConfirmed = false;
	};
	%tradeObject.schedule(30000, checkTradeExpire);
	%tradeObject.schedule(180000, expireTrade);
	
	%client.trade = %tradeObject;
	%target.trade = %tradeObject;
	%client.chatMessage("Trade request sent to" SPC %target.name);
	%target.chatMessage("You've received a trade request from" SPC %client.name SPC "Do /acceptTrade to continue, or /declineTrade");
}

function servercmdDeclineTrade(%client)
{
	if(!isObject(%client.trade))
	{
		%client.chatMessage("You're not in a trade.");
		return;
	}
	
	%trade = %client.trade;
	
	if(%trade.target == %client)
	{
		%trade.target.chatMessage("You have cancelled the trade.");
		%trade.creator.chatMessage(%client.name SPC "cancelled the trade.");
		
		%trade.delete();
	}
	else if(%trade.creator == %client)
	{
		%trade.creator.chatMessage("You have cancelled the trade.");
		%trade.target.chatMessage(%client.name SPC "cancelled the trade.");
		
		%trade.delete();
	}
}

function servercmdAcceptTrade(%client)
{
	if(!isObject(%client.trade))
	{
		%client.chatMessage("You're not in a trade.");
		return;
	}
	
	%trade = %client.trade;
	
	if(%trade.target == %client)
	{
		%trade.creator.chatMessage("You are now trading with" SPC %client.name @ ". Do /offer <slot number> to offer an item in your inventory.");
		%client.chatMessage("You are now trading with" SPC %trade.creator.name @ ". Do /offer <slot number> to offer an item in your inventory.");
		%trade.targetAccepted = true;
	}
	else if(%trade.creator == %client)
	{
		%client.chatMessage("You're the owner of this trade...");
		return;
	}
}

function servercmdOffer(%client, %slot)
{
	if(!isObject(%client.trade))
	{
		%client.chatMessage("You're not in a trade.");
		return;
	}
	
	if(!%client.trade.targetAccepted)
	{
		%client.chatMessage("This trade hasn't started yet.");
		return;
	}
	
	if(!isObject(%client.player))
	{
		%client.chatMessage("You need a player to do this.");
		return;
	}
	
	%slot--;
	%trade = %client.trade;
	
	if(!isObject(%client.player.tool[%slot]))
	{
		%client.chatMessage("You don't have an item in that slot.");
		return;
	}
	
	%client.chatMessage("You are offering your" SPC %client.player.tool[%slot].uiName @ ". Do /confirmTrade when done.");
	
	if(%trade.creator == %client)
	{
		%trade.target.chatMessage(%client.name SPC "is offering their" SPC %client.player.tool[%slot].uiName);
		%trade.creatorOffer[%slot] = %client.player.tool[%slot];
		%trade.creatorItemsOffered++;
		%trade.creatorConfirmed = false;
		%trade.targetConfirmed = false;
	}
	else if(%trade.target == %client)
	{
		%trade.creator.chatMessage(%client.name SPC "is offering their" SPC %client.player.tool[%slot].uiName);
		%trade.targetOffer[%slot] = %client.player.tool[%slot];
		%trade.targetItemsOffered++;
		%trade.creatorConfirmed = false;
		%trade.targetConfirmed = false;
	}
}

function servercmdConfirmTrade(%client)
{
	if(!isObject(%client.trade))
	{
		%client.chatMessage("You're not in a trade.");
		return;
	}
	
	if(!%client.trade.targetAccepted)
	{
		%client.chatMessage("This trade hasn't started yet.");
		return;
	}
	
	%trade = %client.trade;
	%client.chatMessage("You have confirmed your offer.");
	
	if(%trade.creator == %client)
	{
		%trade.target.chatMessage(%client.name SPC "has confirmed their offer.");
		%trade.creatorConfirmed = true;
	}
	else if(%trade.target == %client)
	{
		%trade.creator.chatMessage(%client.name SPC "has confirmed their offer.");
		%trade.targetConfirmed = true;
	}
	
	
	if(%trade.targetConfirmed && %trade.creatorConfirmed)
	{
		for(%i = 0; %i < 50; %i++)
		{
			if(%trade.creatorOffer[%i] == 0)
				continue;
			
			if(%trade.creatorOffer[%i] != %trade.creator.player.tool[%i])
			{
				%trade.creator.chatMessage("Something you offered is missing. Trade cancelled.");
				%trade.target.chatMessage("Something" SPC %trade.creator.name SPC "offered went missing. Trade cancelled.");
				%trade.delete();
				return;
			}
		}
		
		for(%i = 0; %i < 50; %i++)
		{
			if(%trade.targetOffer[%i] == 0)
				continue;
			
			if(%trade.targetOffer[%i] != %trade.target.player.tool[%i])
			{
				%trade.target.chatMessage("Something you offered is missing. Trade cancelled.");
				%trade.creator.chatMessage("Something" SPC %trade.target.name SPC "offered went missing. Trade cancelled.");
				%trade.delete();
				return;
			}
		}
		
		for(%i = 0; %i < 50; %i++)
		{
			if(%trade.creatorOffer[%i] > 0)
			{
				%trade.creator.player.removeTool(%i);
				%trade.target.player.directAddWeapon(%trade.creatorOffer[%i]);
			}
			
			if(%trade.targetOffer[%i] > 0)
			{
				%trade.target.player.removeTool(%i);
				%trade.creator.player.directAddWeapon(%trade.targetOffer[%i]);
			}
		}
		
		%trade.creator.chatMessage("Trade completed.");
		%trade.target.chatMessage("Trade completed.");
		%trade.delete();
	}
}

// A modification of the addItem event I created for Dungeon Generator. The main difference is that 'overflow' items are dropped on the ground rather than being lost. Should help people out. 
function player::directAddWeapon(%player, %itemData)
{
	%item = %itemData;
	%client = %player.client;
	%uiName = %itemData.uiName;

	if(!isObject(%client.player) || %itemData $= "")
		return;
	
	if(!isObject(%item)) //item not found
	{
		%client.chatMessage("ERROR: ITEM" SPC %itemData SPC "NOT FOUND. NOTIFY HOST.");
		return;
	}
	
	if(%item.getClassName() !$= "ItemData")
	{
		%client.chatMessage("ERROR:" SPC %itemData SPC "IS NOT AN ITEMDATA DATABLOCK.");
		return;
	}

	for(%i = 0; %i < %player.dataBlock.maxTools; %i++)
	{
		if(!%player.tool[%i])
		{
			%slot = %i;
			break;
		}
	}

	if(%slot $= "") //no tool slots available
	{
		%dropItem = new Item()
		{
			dataBlock = %item;
			position = %player.getHackPosition();
		};

		%dropItem.BL_ID = %client.BL_ID;
		%dropItem.minigame = %client.minigame;
		%dropItem.spawnBrick = -1;

		%dropItem.schedulePop();
		missionCleanup.add(%dropItem);

		commandToClient(%player.client, 'messageBoxOK', "Notification", "You didn't have enough room in your inventory for that item! It's now on the ground.");
		return;
	}
	
	%player.weaponCount++;
	%player.tool[%slot] = %item;
	messageClient(%client, 'MsgItemPickup', "", %slot, %item);
}

function player::removeTool(%this, %slot)
{
	%this.tool[%slot] = 0;
	%this.weaponCount--;
	messageClient(%this.client, 'MsgItemPickup',' ', %slot, 0);
}